package compbio.cassandra;

import java.util.List;

import org.apache.log4j.Logger;

import com.datastax.driver.core.Row;
import com.datastax.driver.core.Session;
import com.datastax.driver.core.ResultSet;

import compbio.data.sequence.FastaSequence;
import compbio.engine.JpredJob;
import compbio.engine.ProteoCachePropertyHelperManager;
import compbio.util.PropertyHelper;

public class CassandraWriter {
	private Session session;
	private static final PropertyHelper ph = ProteoCachePropertyHelperManager.getPropertyHelper();
	private static Logger log = Logger.getLogger(CassandraNativeConnector.class);

	CassandraWriter() {
		Session inis = CassandraNativeConnector.getSession();
		setSession(inis);
	}

	public void setSession(Session s) {
		assert s != null;
		session = s;
	}

	public boolean JobisNotInsterted(String jobid) {
		ResultSet results1 = session.execute("SELECT * FROM ProteinLog WHERE JobID = '" + jobid + "';");
		if (results1.isExhausted()) {
			return true;
		}
		return false;
	}

	public boolean JobisNotArchived(String jobid) {
		ResultSet results1 = session.execute("SELECT * FROM JpredArchive WHERE JobID = '" + jobid + "';");
		if (results1.isExhausted()) {
			return true;
		}
		return false;
	}

	/*
	 * inserting data into the tables for queries
	 */
	public int FormQueryTables(JpredJob job) {
		if (JobisNotInsterted(job.getJobID())) {
			String id = job.getJobID();
			String protein = job.getProtein();
			String finalstatus = job.getFinalStatus();
			String execstatus = job.getExecutionStatus();

			String com1 = "INSERT INTO ProteinLog (JobID, IP, DataBegin, DataEnd, FinalStatus, ExecutionStatus, Protein) VALUES ('" + id
					+ "','" + job.getIP() + "','" + job.getStartingTimeStr() + "','" + job.getEndTimeStr() + "','" + finalstatus + "','"
					+ execstatus + "','" + protein + "');";
			session.execute(com1);
			if (finalstatus.equals("OK")) {
				String com2 = "INSERT INTO ProteinData " + "(jobtime, JobID, ExecTime, Protein)" + " VALUES (" + job.getStartingDate()
						+ ",'" + id + "'," + job.getExecutionTime() + ",'" + protein + "');";
				session.execute(com2);

				String allpredictions = "";
				List<FastaSequence> pr = job.getPredictions();
				for (FastaSequence pred : pr) {
					String predictionname = pred.getId();
					String prediction = pred.getSequence().replaceAll("\n", "");
					allpredictions += "'" + predictionname + "':'" + prediction + "',";
				}
				String final_prediction = "";
				if (!allpredictions.equals("")) {
					final_prediction = allpredictions.substring(0, allpredictions.length() - 1);
				}

				ResultSet results2 = session.execute("SELECT * FROM ProteinRow WHERE JobID = '" + job.getJobID() + "';");
				if (results2.isExhausted()) {
					String com3 = "INSERT INTO ProteinRow (Protein, JobID, Predictions) VALUES ('" + protein + "','" + id + "',{"
							+ final_prediction + "});";
					session.execute(com3);
				}
			} else {
				String com5 = "INSERT INTO FailLog (jobtime, JobID, ExecTime, ip, FinalStatus) VALUES (" + job.getStartingDate() + ",'"
						+ id + "'," + job.getExecutionTime() + ",'" + job.getIP() + "', '" + finalstatus + "');";
				session.execute(com5);
			}

			// update Main parameters if the job is the earliest job so far
			ResultSet results3 = session.execute("SELECT * FROM MainParameters WHERE Name = 'EarliestJobDate';");
			boolean updateparameter = true;
			if (!results3.isExhausted()) {
				Row r = results3.one();
				if (job.getStartingDate() >= Long.parseLong(r.getString("Value")))
					updateparameter = false;
			}
			if (updateparameter) {
				session.execute("INSERT INTO MainParameters (Name, Value) VALUES ('EarliestJobDate','" + job.getStartingDate() + "');");
			}

			// update internal job counts (used by the Daily Statistics
			// requests)
			//TODO I don't like the bit of code. There should not be so many counters...
			int njobsTotal = 1;
			int njobsOk = 0;
			int njobsStop = 0;
			int njobsError = 0;
			int njobsTimeOut = 0;
			if (finalstatus.equals("OK"))
				njobsOk = 1;
			else if (finalstatus.equals("TIMEDOUT"))
				njobsTimeOut = 1;
			else if (finalstatus.equals("JPREDERROR"))
				njobsError = 1;
			else if (finalstatus.equals("STOPPED"))
				njobsStop = 1;
			ResultSet results4 = session.execute("SELECT * FROM JobDateInfo WHERE jobday = " + job.getStartingDate() + ";");
			if (!results4.isExhausted()) {
				Row r = results4.one();
				njobsTotal += r.getLong("Total");
				njobsOk += r.getLong("TotalOK");
				njobsError += r.getLong("TotalError");
				njobsStop += r.getLong("TotalStopped");
				njobsTimeOut += r.getLong("TotalTimeOut");
			}
			String com = "INSERT INTO JobDateInfo " + "(jobday, Total, TotalOK, TotalStopped, TotalError, TotalTimeOut)" + " VALUES ("
					+ job.getStartingDate() + "," + njobsTotal + "," + njobsOk + "," + njobsStop + "," + njobsError + "," + njobsTimeOut
					+ ");";
			session.execute(com);
			return 1;
		}
		return 0;
	}

	/*
	 * insert data from a real Jpred job: timing+IP, Execution Status, Final
	 * status, protein sequence, predictions, alignment, LOG and tar.gz files
	 */
	public int ArchiveData(JpredJob job, String archivepath) {
		if (JobisNotArchived(job.getJobID())) {
			String id = job.getJobID();
			String log = job.getLog().replaceAll("'", "");
			String com = "INSERT INTO JpredArchive (JobID, Protein, IP, StartTime, ExecTime, FinalStatus, ExecutionStatus, LOG, ArchiveLink) VALUES ('"
					+ id
					+ "','"
					+ job.getProtein()
					+ "','"
					+ job.getIP()
					+ "',"
					+ job.getStartingTime()
					+ ","
					+ job.getExecutionTime()
					+ ",'" + job.getFinalStatus() + "','" + job.getExecutionStatus() + "','" + log + "','" + archivepath + "');";
			session.execute(com);
			List<FastaSequence> predictions = job.getPredictions();
			for (FastaSequence p : predictions) {
				session.execute("UPDATE JpredArchive SET predictions = predictions + {'" + p.getId() + "':'"
						+ p.getSequence().replaceAll("\n", "") + "'} WHERE JobID = '" + id + "';");
			}

			List<FastaSequence> seqs = job.getAlignment();
			for (FastaSequence s : seqs) {
				session.execute("UPDATE JpredArchive SET alignment = alignment + {'" + s.getId() + "':'"
						+ s.getSequence().replaceAll("\n", "") + "'} WHERE JobID = '" + id + "';");
			}
			return 1;
		}
		return 0;
	}

}
